@@ -1,16 +1,20 @@
-import abc
-from past.builtins import long
+import os
 import subprocess
-
 import logging
+
 import requests
+import requests.packages
 from requests import exceptions
-import six
 
-from .util import ApiXmlHelper
-from .exceptions import KempTechApiException, CommandNotAvailableException
-from .exceptions import ConnectionTimeoutException
+from .api_xml import (
+    get_error_msg, is_successful,
+    get_data_field, parse_to_dict)
+from .exceptions import (
+    KempTechApiException,
+    CommandNotAvailableException,
+    ConnectionTimeoutException)
 
+from .exceptions import LoadMasterParameterError
 
 requests.packages.urllib3.disable_warnings()
 log = logging.getLogger(__name__)
@@ -39,6 +43,7 @@ class HttpClient(object):
         :param file: Location of file to send.
         :return: The Status code of request and the response text body.
         """
+        response = None
         cmd_url = "{endpoint}{cmd}?".format(endpoint=self.endpoint,
                                             cmd=rest_command)
         log.debug("Request is: %s", cmd_url)
@@ -76,7 +81,8 @@ class HttpClient(object):
             raise
         except exceptions.HTTPError:
             log.error("A HTTP error occurred with request to %s.", cmd_url)
-            raise KempTechApiException(msg=response.text)
+            raise KempTechApiException(msg=response.text,
+                                       code=response.status_code)
         except exceptions.RequestException:
             log.error("An error occurred with request to %s.", cmd_url)
             raise
@@ -86,64 +92,17 @@ class HttpClient(object):
         return self._do_request('GET', rest_command, parameters)
 
     def _post(self, rest_command, file=None, parameters=None):
-        return self._do_request('POST', rest_command, parameters=parameters, file=file)
+        return self._do_request('POST', rest_command, parameters=parameters,
+                                file=file)
+
+# ---- LoadMaster ---
 
 
-class ComplexIdMixin(object):
-    """Mixin for adding non-trivial IDs."""
-
-    @abc.abstractproperty
-    def id(self):
-        """Must return a dict with unique ID parameters for KEMP API."""
-        raise NotImplementedError("This abstractproperty needs implementation")
-
-    @id.setter
-    @abc.abstractmethod
-    def id(self, value):
-        raise NotImplementedError("This abstractmethod needs implementation")
-
-
-class BaseObjectModel(HttpClient):
-    """A class to build objects based on KEMP RESTful API.
-
-    Subclasses built from this class need to name their parameters
-    the same as their RESTful API counterpart in order for this
-    class to work.
-    """
-
-    @abc.abstractproperty
-    def api_name(self):
-        raise NotImplementedError("This abstractproperty needs implementation")
-
-    def __init__(self, parameters):
-        for api_key, api_value in parameters:
-            self.__dict__[api_key] = api_value
-
-    def save(self):
-        command = 'mod' if self.exists else 'add'
-        self._get_request('%s%s' % (command, self.api_name), self.to_dict())
-
-    def delete(self):
-        self._get_request('%s%s' % ('del', self.api_name), self.id)
-
-    @property
-    def exists(self):
-        return self._get_request('show' + self.api_name, self.id) < 300
-
-    def to_dict(self):
-        """Return a dictionary containing attributes of class.
-
-        Ignore attributes that are set to None or are not a string or int;
-        also ignore endpoint as it is not an API thing.
-        """
-        attributes = {}
-        for attr in self.__dict__:
-            if (self.__dict__[attr] is not None or not
-                    self.__dict__[attr] == 'endpoint' or not
-                    isinstance(self.__dict__[attr], six.string_types) or not
-                    isinstance(self.__dict__[attr], (int, long))):
-                attributes[attr] = self.__dict__[attr]
-        return attributes
+def send_response(response):
+    if is_successful(response):
+        return parse_to_dict(response)
+    else:
+        raise KempTechApiException(get_error_msg(response))
 
 
 class LoadMaster(HttpClient):
@@ -155,6 +114,9 @@ class LoadMaster(HttpClient):
         self.password = password
         self.port = port
 
+    def __repr__(self):
+        return '{}:{}'.format(self.ip_address, self.port)
+
     @property
     def endpoint(self):
         return "https://{user}:{pw}@{ip}:{port}/access".format(
@@ -170,19 +132,20 @@ class LoadMaster(HttpClient):
             'value': value,
         }
         response = self._get('/set', parameters)
-        return ApiXmlHelper.is_successful(response)
+        if not is_successful(response):
+            raise LoadMasterParameterError(self, parameters)
 
     def get_parameter(self, parameter):
         parameters = {
             'param': parameter,
         }
         response = self._get('/get', parameters)
-        value = ApiXmlHelper.get_data_field(response, parameter)
+        value = get_data_field(response, parameter)
         if isinstance(value, dict):
             # This hack converts possible HTML to an awful one string
             # disaster instead of returning parsed html as an OrderedDict.
             value = "".join("{!s}={!r}".format(key, val) for (key, val) in
-                            value.items())
+                            sorted(value.items()))
         return value
 
     def enable_api(self):
@@ -202,69 +165,49 @@ class LoadMaster(HttpClient):
 
     def stats(self):
         response = self._get('/stats')
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def update_firmware(self, file):
         response = self._post('/installpatch', file)
-        return ApiXmlHelper.is_successful(response)
+        return is_successful(response)
 
     def shutdown(self):
         response = self._get('/shutdown')
-        return ApiXmlHelper.is_successful(response)
+        return is_successful(response)
 
     def reboot(self):
         response = self._get('/reboot')
-        return ApiXmlHelper.is_successful(response)
+        return is_successful(response)
 
     def get_sdn_controller(self):
         response = self._get('/getsdncontroller')
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def get_license_info(self):
         try:
             response = self._get('360/licenseinfo')
-            if ApiXmlHelper.is_successful(response):
-                return ApiXmlHelper.parse_to_dict(response)
-            else:
-                raise KempTechApiException(ApiXmlHelper.get_error_msg(
-                    response))
+            send_response(response)
+
         except KempTechApiException:
-            raise CommandNotAvailableException('/access360/licenseinfo')
+            raise CommandNotAvailableException(
+                self, '/access360/licenseinfo')
 
     def list_addons(self):
         response = self._get('/listaddon')
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def upload_template(self, file):
         response = self._post('/uploadtemplate', file)
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def list_templates(self):
         response = self._get('/listtemplates')
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def delete_template(self, template_name):
         params = {'name': template_name}
         response = self._get('/deltemplate', parameters=params)
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def apply_template(self, virtual_ip, port, protocol, template_name):
         params = {
@@ -274,17 +217,11 @@ class LoadMaster(HttpClient):
             'name': template_name,
         }
         response = self._get('/addvs', parameters=params)
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def get_sdn_info(self):
         response = self._get('/sdninfo')
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def restore_backup(self, backup_type, file):
         # 1 LoadMaster Base Configuration
@@ -293,17 +230,31 @@ class LoadMaster(HttpClient):
         if backup_type not in [1, 2, 3]:
             backup_type = 2
         params = {"type": backup_type}
-        response = self._post('/restore'.format(backup_type), file=file,
+        response = self._post('/restore', file=file,
                               parameters=params)
-        if ApiXmlHelper.is_successful(response):
-            return ApiXmlHelper.parse_to_dict(response)
-        else:
-            raise KempTechApiException(ApiXmlHelper.get_error_msg(response))
+        send_response(response)
 
     def backup(self):
         # Dirty API, dirty hack.
-        curl = ['curl', '-k', '{}/backup'.format(self.endpoint)]
+
+        if not os.path.exists('backups'):
+            os.makedirs('backups')
         file_name = "backups/{}.backup".format(self.ip_address)
+
         with open(file_name, 'wb') as file:
+            curl = ['curl', '-k', '{}/backup'.format(self.endpoint)]
             subprocess.call(curl, stdout=file)
         return file_name
+
+    def alsi_license(self, kemp_id, password):
+        params = {
+            "kemp_id": kemp_id,
+            "password": password,
+        }
+        response = self._get('/alsilicense', parameters=params)
+        send_response(response)
+
+    def set_initial_password(self, password):
+        params = {"passwd": password}
+        response = self._get('/set_initial_password', parameters=params)
+        send_response(response)
