@@ -1,34 +1,38 @@
-from .util import ApiXmlHelper
+from .api_xml import get_error_msg
+
+# 400+ shouldn't be handled here, but cover it anyway.
+# This is just optional fallback for
+# laziness when no message is given.
+STATUS_CODES = {
+    400: 'Mandatory parameter missing from request',
+    401: 'Client Error: Authorization required',
+    403: 'Incorrect permissions',
+    404: 'Not found. Ensure the API is enabled on the LoadMaster',
+    405: 'Unknown command',
+    422: 'Invalid operation'
+}
+
+
+def get_api_exception_message(msg, code, is_xml_msg):
+    if msg is not None:
+        # 400 Errors should be handled here. This will pass the error
+        # message given by the LoadMaster API and show it as the exception
+        # message in the traceback.
+        message = get_error_msg(msg) if is_xml_msg else msg
+    else:
+        try:
+            message = '{} {}.'.format(code, STATUS_CODES[code])
+        except KeyError:
+            message = "An unknown error has occurred ({}).".format(code)
+
+    return message
 
 
 class KempTechApiException(Exception):
     """Raised if HTTP request has failed."""
 
     def __init__(self, msg=None, code=None, is_xml_msg=True):
-        if msg is not None:
-            # 400 Errors should be handled here. This will pass the error
-            # message given by the LoadMaster API and show it as the exception
-            # message in the traceback.
-            message = ApiXmlHelper.get_error_msg(msg) if is_xml_msg else msg
-        else:
-            # 400+ shouldn't be handled here, but cover it anyway.
-            # This is just optional fallback for
-            # laziness when no message is given.
-            if code == 400:
-                message = "400 Mandatory parameter missing from request."
-            elif code == 401:
-                message = "401 Client Error: Authorization required."
-            elif code == 403:
-                message = "403 Incorrect permissions."
-            elif code == 404:
-                message = ("404 Not found. "
-                           "Ensure the API is enabled on the LoadMaster.")
-            elif code == 405:
-                message = "405 Unknown command."
-            elif code == 422:
-                message = "422 Invalid operation."
-            else:
-                message = "An unknown error has occurred."
+        message = get_api_exception_message(msg, code, is_xml_msg)
         super(KempTechApiException, self).__init__(message)
 
 
@@ -70,23 +74,33 @@ class ApiNotEnabledError(KempTechApiException):
 
 
 class CommandNotAvailableException(KempTechApiException):
-    def __init__(self, cmd_name):
-        msg = "Command '{}' is not available on the LoadMaster.".format(
-            cmd_name)
+    def __init__(self, lm, cmd_name):
+        msg = "Command '{}' is not available on LoadMaster {}.".format(
+            cmd_name, lm)
         super(CommandNotAvailableException, self).__init__(msg,
                                                            is_xml_msg=False)
 
 
-class InvalidOperationException(KempTechApiException):
-    def __init__(self, cmd_name):
-        msg = "Command '{}' is not available on the LoadMaster.".format(
-            cmd_name)
-        super(InvalidOperationException, self).__init__(msg,
-                                                        is_xml_msg=False)
-
-
 class ConnectionTimeoutException(KempTechApiException):
     def __init__(self, lm):
         msg = "Connection timed out to '{}'.".format(lm)
-        super(ConnectionTimeoutException, self).__init__(msg,
-                                                         is_xml_msg=False)
+        super(ConnectionTimeoutException, self).__init__(
+            msg, is_xml_msg=False)
+
+
+def get_parameter_message(lm, parameters):
+    try:
+        param = parameters['param']
+        value = parameters['value']
+        msg = '{} failed to set {}: {}'.format(lm, param, value)
+    except (KeyError, TypeError) as e:
+        msg = '{} failed to set {} ({})'.format(
+            lm, parameters, str(e))
+
+    return msg
+
+
+class LoadMasterParameterError(Exception):
+    def __init__(self, lm, parameters):
+        msg = get_parameter_message(lm, parameters)
+        super(LoadMasterParameterError, self).__init__(msg)
